#version 430 core
out vec4 FragColor;
in vec2 TexCoords;

uniform sampler2D screenTexture;
uniform sampler2D normalTexture;
uniform sampler2D depthTexture;
uniform sampler2D depthMap;

const float offset = 0.0007;
const float threshold = 0.1; // Adjust this threshold as needed

void main()
{
    vec2 offsets[9] = vec2[](
        vec2(-offset,  offset), 
        vec2( 0.0,    offset), 
        vec2( offset,  offset), 
        vec2(-offset,  0.0),  
        vec2( 0.0,    0.0),  
        vec2( offset,  0.0),  
        vec2(-offset, -offset), 
        vec2( 0.0,   -offset), 
        vec2( offset, -offset)
    );

    vec3 sampleTex[9];
    vec3 centerNormal = texture(normalTexture, TexCoords).rgb;
    vec3 centerDepth = texture(depthTexture, TexCoords).rgb;
    float thresholdNormal = 0.5f;
    float thresholdDepth = 0.01f;
    vec3 edgeColor = vec3(0.0, 0.0, 0.0);
    for(int i = 0; i < 9; i++)
    {
        sampleTex[i] = texture(normalTexture, TexCoords.st + offsets[i]).rgb;
        float differenceNormal = length(centerNormal - sampleTex[i]);

        if (differenceNormal > thresholdNormal) {
            FragColor = vec4(edgeColor, 1.0);
            return;
        }
        sampleTex[i] = texture(depthTexture, TexCoords.st + offsets[i]).rgb;
        float differenceDepth = length(centerDepth - sampleTex[i]);

        if (differenceDepth > thresholdDepth) {
            FragColor = vec4(edgeColor, 1.0);
            return;
        }
    }
    float depth = texture(depthMap, TexCoords).r;
    vec3 col = texture(screenTexture, TexCoords).rgb;
    FragColor = vec4(col, 1.0);
}